#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :   tagHelper.py
@Time    :   2019/07/18
@Author  :   Yaronzz 
@Version :   2.0
@Contact :   yaronhuang@foxmail.com
@Desc    :  
"""
import os

import requests
from mutagen import File
from mutagen import flac
from mutagen import mp4
from mutagen.id3 import TALB, TCOP, TDRC, TIT2, TPE1, TRCK, APIC, TCON, TCOM, TSRC, USLT


def __extension__(filepath: str):
    index = filepath.rfind('.')
    ret = filepath[index + 1:len(filepath)]
    return str.lower(ret)


def __content__(filepath: str):
    if filepath is None:
        return None
    if 'http' in filepath:
        re = requests.get(filepath)
        return re.content
    try:
        with open(filepath, "rb") as f:
            data = f.read()
            return data
    except:
        return None


def __tryInt__(obj):
    try:
        ret = int(obj)
        return ret
    except:
        return 0


def __tryStr__(obj):
    return obj if obj is not None else ""


def __tryList__(obj):
    if obj is None or len(obj) <= 0:
        return ''
    return ", ".join(obj)


class TagTool(object):
    def __init__(self, filePath):
        if os.path.isfile(filePath) is False:
            return

        self._filepath = filePath
        self._ext = __extension__(filePath)
        self._handle = File(filePath)

        self.title = ''
        self.album = ''
        self.albumartist = ''
        self.artist = ''
        self.copyright = ''
        self.tracknumber = ''
        self.totaltrack = ''
        self.discnumber = ''
        self.totaldisc = ''
        self.genre = ''
        self.date = ''
        self.composer = ''
        self.isrc = ''
        self.lyrics = ''

        self.__load__()

    def save(self, coverPath: str = None):
        try:
            if 'mp3' in self._ext:
                return self.__saveMp3__(coverPath)
            if 'flac' in self._ext:
                return self.__saveFlac__(coverPath)
            if 'mp4' in self._ext or 'm4a' in self._ext:
                return self.__saveMp4__(coverPath)
            return False
        except Exception as e:
            return False, str(e)

    def addPic(self, converPath: str = None):
        try:
            self.__savePic__(converPath)
            self._handle.save()
            return True
        except Exception as e:
            return False, str(e)

    def addLyrics(self, lyrics: str = None):
        try:
            if 'mp3' in self._ext:
                self._handle.tags.add(USLT(encoding=3, lang=u'eng', desc=u'desc', text=__tryStr__(self.lyrics)))
            if 'flac' in self._ext:
                self._handle.tags['lyrics'] = __tryStr__(self.lyrics)
            if 'mp4' in self._ext or 'm4a' in self._ext:
                self._handle.tags['\xa9lyr'] = __tryStr__(self.lyrics)
            self._handle.save()
            return True
        except Exception as e:
            return False, str(e)

    def __load__(self):
        try:
            if 'mp3' in self._ext:
                return self.__getMp3__()
            if 'flac' in self._ext:
                return self.__getFlac__()
            if 'mp4' in self._ext or 'm4a' in self._ext:
                return self.__getMp4__()
        except:
            return

    def __saveMp3__(self, coverPath):
        if self._handle.tags is None:
            self._handle.add_tags()
        self._handle.tags.add(TIT2(encoding=3, text=self.title))
        self._handle.tags.add(TALB(encoding=3, text=self.album))
        # self._handle.tags.add(TOPE(encoding=3, text=self.albumartist))
        self._handle.tags.add(TPE1(encoding=3, text=self.artist))
        self._handle.tags.add(TCOP(encoding=3, text=self.copyright))
        self._handle.tags.add(TRCK(encoding=3, text=str(self.tracknumber)))
        # self._handle.tags.add(TRCK(encoding=3, text=self.discnum))
        self._handle.tags.add(TCON(encoding=3, text=self.genre))
        self._handle.tags.add(TDRC(encoding=3, text=self.date))
        self._handle.tags.add(TCOM(encoding=3, text=self.composer))
        self._handle.tags.add(TSRC(encoding=3, text=self.isrc))
        self._handle.tags.add(USLT(encoding=3, lang=u'eng', desc=u'desc', text=self.lyrics))
        self.__savePic__(coverPath)
        self._handle.save()
        return True

    def __getMp3__(self):
        if self._handle.tags is None:
            self._handle.add_tags()

    def __saveFlac__(self, coverPath):
        if self._handle.tags is None:
            self._handle.add_tags()
        self._handle.tags['title'] = self.title
        self._handle.tags['album'] = self.album
        self._handle.tags['albumartist'] = __tryList__(self.albumartist)
        self._handle.tags['artist'] = __tryList__(self.artist)
        self._handle.tags['copyright'] = __tryStr__(self.copyright)
        self._handle.tags['tracknumber'] = str(self.tracknumber)
        self._handle.tags['tracktotal'] = str(self.totaltrack)
        self._handle.tags['discnumber'] = str(self.discnumber)
        self._handle.tags['disctotal'] = str(self.totaldisc)
        self._handle.tags['genre'] = __tryStr__(self.genre)
        self._handle.tags['date'] = __tryStr__(self.date)
        self._handle.tags['composer'] = __tryStr__(self.composer)
        self._handle.tags['isrc'] = __tryStr__(self.isrc)
        self._handle.tags['lyrics'] = __tryStr__(self.lyrics)
        self.__savePic__(coverPath)
        self._handle.save()
        return True

    def __getFlac__(self):
        if self._handle.tags is None:
            return
        self.title = self.__getTagItem__('title')
        self.album = self.__getTagItem__('album')
        self.albumartist = self.__getTagItem__('albumartist')
        self.artist = self.__getTagItem__('artist')
        self.copyright = self.__getTagItem__('copyright')
        self.tracknumber = self.__getTagItem__('tracknumber')
        self.totaltrack = self.__getTagItem__('tracktotal')
        self.discnumber = self.__getTagItem__('discnumber')
        self.totaldisc = self.__getTagItem__('disctotal')
        self.genre = self.__getTagItem__('genre')
        self.date = self.__getTagItem__('date')
        self.composer = self.__getTagItem__('composer')
        self.isrc = self.__getTagItem__('isrc')
        self.lyrics = self.__getTagItem__('lyrics')

    def __saveMp4__(self, coverPath):
        self._handle.tags['\xa9nam'] = self.title
        self._handle.tags['\xa9alb'] = self.album
        self._handle.tags['aART'] = __tryList__(self.albumartist)
        self._handle.tags['\xa9ART'] = __tryList__(self.artist)
        self._handle.tags['cprt'] = __tryStr__(self.copyright)
        self._handle.tags['trkn'] = [[__tryInt__(self.tracknumber), __tryInt__(self.totaltrack)]]
        self._handle.tags['disk'] = [[__tryInt__(self.discnumber), __tryInt__(self.totaldisc)]]
        self._handle.tags['\xa9gen'] = __tryStr__(self.genre)
        self._handle.tags['\xa9day'] = __tryStr__(self.date)
        self._handle.tags['\xa9wrt'] = __tryList__(self.composer)
        self._handle.tags['\xa9lyr'] = __tryStr__(self.lyrics)
        self.__savePic__(coverPath)
        self._handle.save()
        return True

    def __getMp4__(self):
        self.title = self.__getTagItem__('\xa9nam')
        self.album = self.__getTagItem__('\xa9alb')
        self.albumartist = self.__getTagItem__('aART')
        self.artist = self.__getTagItem__('\xa9ART')
        self.copyright = self.__getTagItem__('\cprt')
        self.tracknumber = self.__getTagItem__('trkn')
        self.totaltrack = self.__getTagItem__('trkn')
        self.discnumber = self.__getTagItem__('disk')
        self.totaldisc = self.__getTagItem__('disk')
        self.genre = self.__getTagItem__('\xa9gen')
        self.date = self.__getTagItem__('\xa9day')
        self.composer = self.__getTagItem__('\xa9wrt')
        self.lyrics = self.__getTagItem__('\xa9lyr')

    def __getTagItem__(self, name):
        if name in self._handle.tags:
            return self._handle.tags[name]
        return ''

    def __savePic__(self, coverPath):
        data = __content__(coverPath)
        if data is None:
            return

        if 'flac' in self._ext:
            pic = flac.Picture()
            pic.data = data
            if '.jpg' in coverPath:
                pic.mime = u"image/jpeg"
            self._handle.clear_pictures()
            self._handle.add_picture(pic)

        if 'mp3' in self._ext:
            self._handle.tags.add(APIC(encoding=3, data=data))

        if 'mp4' in self._ext or 'm4a' in self._ext:
            pic = mp4.MP4Cover(data)
            self._handle.tags['covr'] = [pic]



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :   jsonHelper.py
@Time    :   2020/08/10
@Author  :   Yaronzz
@Version :   2.0
@Contact :   yaronhuang@foxmail.com
@Desc    :  
"""
from aigpy.dictHelper import DictTool


class ModelBase(object):
    pass


def modelToDict(model):
    if model is None:
        return None
    if not __isModel__(model):
        return None

    members = [attr for attr in dir(model) if not callable(getattr(model, attr)) and not attr.startswith("_")]

    pr = {}
    for name in members:
        value = getattr(model, name)
        if name[0] == '_':
            continue
        if callable(value):
            continue
        if __isModelList__(value):
            value = modelListToDictList(value)
        if __isModel__(value):
            value = modelToDict(value)
        pr[name] = value
    return pr


def dictToModel(indict, model):
    if indict is None or model is None:
        return None
    ret = model
    maps = DictTool(indict)

    members = [attr for attr in dir(ret) if not callable(getattr(model, attr)) and not attr.startswith("_")]

    for key in members:
        if key.lower() not in maps:
            if __isObject__(getattr(ret, key)):
                setattr(ret, key, None)
            continue

        # 判断是否为字典数组
        lvalue = maps[key.lower()]
        if __isDictList__(lvalue):
            value = dictListToModelList(lvalue, getattr(ret, key))
        # 判断是否为字典
        elif __isDict__(lvalue):
            value = dictToModel(lvalue, getattr(ret, key))
        else:
            value = lvalue

        setattr(ret, key, value)
    return ret


def dictListToModelList(jList, model):
    if jList is None or model is None:
        return jList
    ret = []
    for item in jList:
        data = dictToModel(item, model)
        ret.append(data)
    return ret


def modelListToDictList(mList):
    if mList is None:
        return mList
    ret = []
    for item in mList:
        data = modelToDict(item)
        ret.append(data)
    return ret


def __isDict__(data):
    if isinstance(data, dict):
        return True
    return False


def __isObject__(data):
    if isinstance(data, object):
        return True
    return False


def __isDictList__(data):
    if isinstance(data, list):
        for item in data:
            return __isDict__(item)
    return False


def __isModelList__(data):
    if isinstance(data, list):
        return True
    return False


def __isModel__(data):
    return isinstance(data, ModelBase)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# coding:utf-8
"""
@File    :   ffmpegHelper.py
@Time    :   2018/12/17
@Author  :   Yaron Huang 
@Version :   1.0
@Contact :   yaronhuang@qq.com
@Desc    :  
"""
import asyncio
import subprocess
import sys


def convert(srcPath, descPath, bitrate: int = 1600):
    command = (
        'ffmpeg -v quiet -y -i "%s" -acodec libmp3lame -abr true '
        f"-b:a {bitrate} "
        '-af "apad=pad_dur=2, dynaudnorm, loudnorm=I=-17" "%s"'
    )

    if sys.platform == "win32":
        formattedCommand = command % (
            str(srcPath),
            str(descPath),
        )
    else:
        formattedCommand = command % (
            str(srcPath).replace("$", r"\$"),
            str(descPath).replace("$", r"\$"),
        )

    process = subprocess.Popen(
        formattedCommand,
        stdout=asyncio.subprocess.PIPE,
        stderr=asyncio.subprocess.PIPE,
    )

    _, proc_err = process.communicate()

    if process.returncode != 0:
        message = "Convert failed." + proc_err.decode('utf-8')
        return False, message

    return True, ""


def isEnable():
    try:
        process = subprocess.Popen(
            ['ffmpeg', '-version'],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            encoding="utf-8"
        )
    except FileNotFoundError:
        return False

    return True

# t = isEnable()
# convert('e://test.webm', 'e://test.mp3', 159489)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  musicHelp.py
@Date    :  2021/05/13
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :  
"""
from aigpy.modelHelper import ModelBase


class MusicBase(ModelBase):
    id = ''
    name = ''


class User(MusicBase):
    pass


class Artist(MusicBase):
    pass


class Track(MusicBase):
    pass


class Album(MusicBase):
    artists = Artist()
    tracks = Track()


class Show(MusicBase):
    pass


class Playlist(MusicBase):
    isOwn = False
    tracks = Track()

# class AccountData(object):
#     def __init__(self, dbPath: str):
#         self.db = sqlite3.connect(dbPath)

#     def __creatFollowTable__(self):



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  downloadHelper.py
@Date    :  2021/02/07
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :  
"""
import abc
import requests

from aigpy.LockHelper import RWLock
from aigpy.fileHelper import createEmptyFile
from aigpy.memoryHelper import convert, Unit, unitFix
from aigpy.progressHelper import ProgressTool
from aigpy.threadHelper import ThreadTool


class __Part__(object):
    def __init__(self, url: str, requestOffset: int, requestLength: int, fileOffset: int):
        self.url = url
        self.requestOffset = requestOffset
        self.requestLength = requestLength
        self.fileOffset = fileOffset


class UserProgress(metaclass=abc.ABCMeta):
    def __init__(self):
        self.maxNum = 100
        self.curNum = 0
        
    def setMaxNum(self, maxNum: int):
        self.maxNum = maxNum
        self.updateMaxNum()

    def addCurNum(self, num: int):
        self.curNum += num
        self.updateCurNum()
    
    def setCurNum(self, num: int):
        self.curNum = num
        self.updateCurNum()
        
    @abc.abstractmethod
    def updateCurNum(self):
        pass
    
    @abc.abstractmethod
    def updateMaxNum(self):
        pass


def __downloadPartFile__(part: __Part__, parent, progress, unit, retry=3):
    error = None
    while retry > 0:
        retry -= 1
        try:
            rang = 'bytes=%s-%s' % (part.requestOffset, part.requestOffset + part.requestLength - 1)
            headers = {'Range': rang}

            res = requests.get(part.url, headers=headers, timeout=(5, 30), proxies=parent.proxies)
            res.raise_for_status()
        except Exception as e:
            error = e
            continue

        content = res.content

        parent.lock.write_acquire()
        try:
            with open(parent.filePath, 'rb+') as f:
                f.seek(part.fileOffset)
                f.write(content)
            if progress is not None:
                progress.addCurCount(convert(part.requestLength, Unit.BYTE, unit))
            if parent.userProgress is not None:
                parent.userProgress.addCurNum(part.requestLength)
            parent.curSize += part.requestLength
        except Exception as e:
            error = e
            parent.lock.write_release()
            continue
        parent.lock.write_release()
        
        return True, ""
    return False, error


class DownloadTool(object):
    def __init__(self, filePath: str, fileUrls: list, proxies: dict = None):
        self.filePath = filePath
        self.fileUrls = fileUrls
        self.proxies = proxies
        self.partSize = 1048576
        
        self.maxSize = 0
        self.curSize = 0
        
        self.userProgress = None
        self.lock = RWLock()
    
    def __getSize__(self, url):
        ret = requests.head(url, proxies=self.proxies)
        if 'Content-Length' in ret.headers:
            return int(ret.headers['Content-Length'])
        return -1

    def __getUrlsSize__(self, urls):
        totalSize = 0
        array = []
        for item in urls:
            size = self.__getSize__(item)
            totalSize += size
            if size < 0:
                return -1, []
            array.append(size)
        return totalSize, array

    def __getOneUrlParts__(self, url):
        fileSize = self.__getSize__(url)
        if fileSize <= 0:
            return 0, [], "Get file size failed."

        offset = 0
        parts = []
        length = fileSize
        while length > 0:
            if length > self.partSize:
                bf = __Part__(url, offset, self.partSize, offset)
            else:
                bf = __Part__(url, offset, length, offset)
            parts.append(bf)
            offset += self.partSize
            length -= self.partSize
        return fileSize, parts, ""

    def __getMoreUrlsParts__(self, urls):
        fileSize, urlSizes = self.__getUrlsSize__(urls)
        if fileSize <= 0:
            return 0, [], "Get some file sizes failed."

        parts = []
        offset = 0
        for i, url in enumerate(urls):
            parts.append(__Part__(url, 0, urlSizes[i], offset))
            offset += urlSizes[i]
        return fileSize, parts, ""

    def setUserProgress(self, progress):
        self.userProgress = progress
        
    def setPartSize(self, size: int):
        self.partSize = size
    
    def start(self, showProgress: bool = False, threadNum: int = 10) -> (bool, str):
        size = len(self.fileUrls)
        if size == 1:
            fileSize, parts, msg = self.__getOneUrlParts__(self.fileUrls[0])
        elif size > 1:
            fileSize, parts, msg = self.__getMoreUrlsParts__(self.fileUrls)
        else:
            return False, "Urls is empty."

        if msg != "":
            return False, msg
        
        self.maxSize = fileSize

        try:
            check = createEmptyFile(self.filePath, fileSize)
            if not check:
                return False, "Create file failed."

            if self.userProgress is not None:
                self.userProgress.setMaxNum(fileSize)
                
            # thread
            threads = ThreadTool(threadNum)
            fileSize, unit = unitFix(fileSize, Unit.BYTE, Unit.MB)

            # Progress
            progress = None
            if showProgress:
                progress = ProgressTool(fileSize, 15, unit=unit.name)

            for item in parts:
                threads.start(__downloadPartFile__, item, self,progress, unit, 3)
            results = threads.waitAll()
            threads.close()

            for item in results:
                if item[0] is False:
                    return False, "Some parts download failed." + item[1]
            return True, ""
        except Exception as e:
            return False, str(e)





#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  memoryHelper.py
@Date    :  2021/8/4
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :
"""
from enum import Enum

OneK = 1024
OneM = OneK * OneK
OneG = OneM * OneK
OneT = OneG * OneK
OneP = OneT * OneK
OneE = OneP * OneK


class Unit(Enum):
    BYTE = 0
    KB = 1
    MB = 2
    GB = 3
    TB = 4


def convert(num: float, srcUnit: Unit, desUnit: Unit) -> float:
    """Convert memory unit, support gb/mb/kb/byte

    Args:
        num (float): value
        srcUnit (MemoryUnit): Original unit
        desUnit (MemoryUnit): Target unit
    """
    if srcUnit == desUnit:
        return num

    try:
        num = float(num)
        if num == 0:
            return 0
        srcIndex = srcUnit.value
        desIndex = desUnit.value
        diff = abs(srcIndex - desIndex)

        while diff != 0:
            if srcIndex > desIndex:
                num = num * 1024
            else:
                num = num / 1024
            diff -= 1

        return num
    except ValueError:
        return 0


def unitFix(num: float, srcUnit: Unit = Unit.BYTE, maxUnit: Unit = Unit.GB) -> (float, Unit):
    """Automatic conversion to appropriate units"""
    try:
        num = float(num)
        if num == 0:
            return 0, srcUnit

        index = srcUnit.value
        while index < maxUnit.value:
            newNum = num / 1024
            if newNum < 1:
                break
            num = newNum
            index += 1

        return num, Unit(index)
    except ValueError:
        return 0, srcUnit



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :   threadHelper.py
@Time    :   2018/12/17
@Author  :   Yaronzz 
@Version :   2.0
@Contact :   yaronhuang@foxmail.com
@Desc    :   Thread Tool 
"""

from concurrent.futures import ThreadPoolExecutor
from concurrent.futures import as_completed


class ThreadTool(object):
    def __init__(self, maxThreadNum: int):
        self.allTask = []
        self.thread = ThreadPoolExecutor(max_workers=maxThreadNum)

    def start(self, function, *args, **kwargs):
        if len(args) > 0 and len(kwargs) > 0:
            handle = self.thread.submit(function, *args, **kwargs)
        elif len(args) > 0:
            handle = self.thread.submit(function, *args)
        elif len(kwargs) > 0:
            handle = self.thread.submit(function, **kwargs)
        else:
            handle = self.thread.submit(function)

        self.allTask.append(handle)
        return handle

    def isFinish(self, handle):
        return handle.done()

    def getResult(self, handle):
        return handle.result()

    def waitAll(self):
        array = []
        for future in as_completed(self.allTask):
            data = future.result()
            array.append(data)
        return array

    def waitAnyone(self):
        as_completed(self.allTask)

    def close(self):
        self.thread.shutdown(False)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :   stringHelper.py
@Time    :   2019/03/11
@Author  :   Yaronzz 
@Version :   2.0
@Contact :   yaronhuang@foxmail.com
@Desc    :  
"""


def isNull(word):
    if word is None or word == "":
        return True
    return False


def isNotNull(word):
    if isNull(word):
        return False
    return True


def isChinese(word, checkPunctuation=False):
    punctuationStr = '，。！？【】（）％＃＠＆１２３４５６７８９０：'
    for ch in word:
        if '\u4e00' <= ch <= '\u9fff':
            return True
        if checkPunctuation and punctuationStr.find(ch) != -1:
            return True
    return False


def convertPunctuationToEnglish(word):
    table = {ord(f): ord(t) for f, t in zip(
        u'，。！？【】（）％＃＠＆１２３４５６７８９０：',
        u',.!?[]()%#@&1234567890:')}
    ret = word.translate(table)
    return ret


def align(string, num, isLeft=True):
    leng = 0
    for c in string:
        leng += 1
        if isChinese(c, True):
            leng += 1

    if leng >= num:
        return string

    appendStr = ""
    while num - leng > 0:
        appendStr += " "
        num -= 1
    if isLeft:
        return string + appendStr
    else:
        return appendStr + string


def getSubOnlyStart(string, start):
    if start not in string:
        return ""
    index = string.index(start)
    return string[index + len(start):]


def getSubOnlyEnd(string, end):
    if end not in string:
        return ""
    index = string.index(end)
    return string[0:index]


def getSub(string, start=None, end=None):
    if start is None and end is None:
        return string
    if end is None:
        return getSubOnlyStart(string, start)
    if start is None:
        return getSubOnlyEnd(string, end)

    string = getSubOnlyStart(string, start)
    if string == "":
        return ""

    ret = getSubOnlyEnd(string, end)
    if ret == "":
        ret = string
    return ret



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :   progressHelper.py
@Time    :   2018/12/28
@Author  :   Yaronzz 
@Version :   2.0
@Contact :   yaronhuang@foxmail.com
@Desc    :   Show ProgressBar
"""

import sys
import threading


class ProgressTool(object):
    def __init__(self, maxCount, barLength=50, icon='▓', unit='', desc=''):
        self.curCount = 0  # 当前计数
        self.maxCount = maxCount  # 最大数量
        self.barLength = barLength  # 进度条长度
        self.icon = icon  # 进度符号
        self.mutex = threading.Lock()  # 互斥锁
        self.isFinish = False
        self.unit = unit
        self.desc = ''
        if len(desc) > 0:
            self.desc = '(' + desc + ')'

    def reset(self, maxCount):
        if self.mutex.acquire():
            self.curCount = 0
            self.maxCount = maxCount
            self.isFinish = False
            self.mutex.release()

    def setCurCount(self, curCount):
        if self.mutex.acquire():
            if self.isFinish is False:
                if curCount >= self.maxCount:
                    curCount = self.maxCount
                    self.isFinish = True
                self.curCount = curCount
                self.__show__()
            self.mutex.release()

    def addCurCount(self, addCount):
        count = self.curCount + addCount
        self.setCurCount(count)

    def step(self):
        count = self.curCount + 1
        self.setCurCount(count)

    def __show__(self):
        try:
            # 计算显示几个进度块
            numBlock = int(self.curCount * self.barLength / self.maxCount)  # 计算显示多少个'>'
            # 计算显示几个空格
            numEmpty = self.barLength - numBlock
            # 计算百分比
            percent = self.curCount * 100.0 / self.maxCount
            # 输出字符串
            process = '%3d' % percent + '%|'
            process += self.icon * numBlock + ' ' * numEmpty + '| '
            process += str(round(self.curCount, 2)) + '/'
            process += str(round(self.maxCount, 2)) + ' ' + self.unit + self.desc

            # 判断是否要换行
            process += '\r' if self.curCount < self.maxCount else '\n'

            sys.stdout.write(process)
            sys.stdout.flush()
        except:
            pass



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :   pipHelper.py
@Time    :   2019/03/11
@Author  :   Yaronzz 
@Version :   2.0
@Contact :   yaronhuang@foxmail.com
@Desc    :   pip server tool
"""

import aigpy.netHelper as netHelper


def getInfo(projectName):
    """Get project information from pypi
    - Return: json or None                              
    """
    url = 'https://pypi.org/pypi/' + projectName + '/json'
    ret = netHelper.downloadJson(url, None)
    return ret


def getLastVersion(projectName):
    """Get project version from pypi
    - Return: str or None                              
    """
    try:
        ret = getInfo(projectName)
        if ret is None:
            return None
        return ret['info']['version']
    except:
        return None


def getVersionList(projectName):
    """Get project all versions from pypi
    - Return: json or None                              
    """
    try:
        ret = getInfo(projectName)
        if ret is None:
            return None
        return ret['releases']
    except:
        return None



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  convertHelper.py
@Date    :  2020/11/16
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :  
"""
from aigpy.memoryHelper import Unit, unitFix


def getMemoryUnitString(num: float, unit: Unit) -> str:
    """Convert the unit to string, eg: num(2653)unit(MB) -> '2.59 GB'

    Args:
        num (float): value
        unit (MemoryUnit): Original unit

    Returns:
        [string]: target string, keep two decimal places
    """
    value, newUnit = unitFix(num, unit)
    string = str(round(value, 2)) + ' ' + newUnit.name
    return string



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  time.py
@Date    :  2021/05/25
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :  
"""


def strToSecond(timeStr: str, splitC: str = ":") -> int:
    array = timeStr.split(splitC)
    if len(array) == 1:
        return int(array[0])
    elif len(array) == 2:
        return int(array[0]) * 60 + int(array[1])
    elif len(array) == 3:
        return int(array[0]) * 3600 + int(array[1]) * 60 + int(array[2])
    return 0



#!/usr/bin/env python
# -*- encoding: utf-8 -*-

"""
@File    :   systemHelper.py
@Time    :   2018/12/20
@Author  :   Yaronzz 
@Version :   2.0
@Contact :   yaronhuang@foxmail.com
@Desc    :  
"""
import platform


def isWindows():
    sysName = platform.system()
    return sysName == "Windows"


def isLinux():
    sysName = platform.system()
    return sysName == "Linux"


def cmpVersion(ver1: str, ver2: str):
    array1 = ver1.split('.')
    array2 = ver2.split('.')
    iIndex = 0
    for obj in array1:
        if len(array2) <= iIndex:
            break
        if int(obj) > int(array2[iIndex]):
            return 1
        if int(obj) < int(array2[iIndex]):
            return -1
        iIndex = iIndex + 1
    return 0



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :   fileHelper.py
@Time    :   2019/03/11
@Author  :   Yaronzz 
@Version :   2.0
@Contact :   yaronhuang@foxmail.com
@Desc    :  
"""

import hashlib
import json
import os

from aigpy.pathHelper import getDirName, mkdirs


def getSize(path: str) -> int:
    if not os.path.isfile(path):
        return 0
    return os.path.getsize(path)


def getHash(path: str, blockSize: int = 2 * 1024 * 1024) -> str:
    with open(path, 'rb') as f:
        sha1 = hashlib.sha1()
        while True:
            data = f.read(blockSize)
            if not data:
                break
            sha1.update(data)
        return sha1.hexdigest()


def getMD5(path: str):
    m = hashlib.md5()
    with open(path, 'rb') as fp:
        while True:
            data = fp.read(4096)
            if not data:
                break
            m.update(data)

    return m.hexdigest()


def getContent(path: str, isBin: bool = False, encoding: str = None):
    size = getSize(path)
    if size <= 0:
        return ""

    mode = 'r' if not isBin else "rb"
    with open(path, mode, encoding=encoding) as fd:
        content = fd.read(size)
    return content


def getJson(path, encoding: str = None):
    try:
        with open(path, 'r', encoding=encoding) as f:
            text = f.read()
            return json.loads(text)
    except:
        return {}


def getLines(path: str, encoding: str = None) -> list:
    content = getContent(path, False, encoding)
    if content == "":
        return []
    return content.split('\n')


def write(path: str, content, mode: str, encoding: str = None):
    try:
        with open(path, mode, encoding=encoding) as fd:
            fd.write(content)
        return True
    except:
        return False


def writeLines(path: str, lines: list, mode: str, encoding: str = None):
    content = "\n".join(lines)
    return write(path, content, mode, encoding)


def writeJson(path: str, data: dict, encoding: str = None):
    try:
        with open(path, 'w', encoding=encoding) as f:
            f.write(json.dumps(data))
            f.flush()
        return True
    except:
        return False


def createEmptyFile(filePath: str, size: int):
    try:
        # Create dir
        path = getDirName(filePath)
        if mkdirs(path) is False:
            return False

        # Create empty file
        fp = open(filePath, "wb")
        fp.truncate(size)
        fp.close()
        return True
    except:
        return False



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :   m3u8Helper.py
@Time    :   2019/08/23
@Author  :   Yaronzz
@Version :   2.0
@Contact :   yaronhuang@foxmail.com
@Desc    :
"""
import re

from aigpy import netHelper
from aigpy.downloadHelper import DownloadTool


def parseUrl(url: str) -> list:
    """Get ts-urls from m3u8-url"""
    content = netHelper.downloadString(url, None)
    return parseTsUrls(content)


def parseTsUrls(m3u8Content: str):
    if m3u8Content is None:
        return []
    
    pattern = re.compile(r"(?<=http).+?(?=\\n)")
    plist = pattern.findall(str(m3u8Content))
    urlList = ["http" + item for item in plist]
    return urlList


def downloadByTsUrls(urls, descPath: str, threadNum: int = 15):
    if len(urls) <= 0:
        return False, "Parse m3u8 url failed."
    
    tool = DownloadTool(descPath, urls)
    check, msg = tool.start(True, threadNum)
    if not check:
        return False, msg

    return True, ""


def download(url: str, descPath: str, threadNum: int = 15):
    """Download file by m3u8-url"""
    urlList = parseUrl(url)
    return downloadByTsUrls(urlList, descPath, threadNum)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :   cmdHelper.py
@Time    :   2019/02/27
@Author  :   Yaronzz 
@Version :   2.2
@Contact :   yaronhuang@foxmail.com
@Desc    :   
"""
import os
import sys
from enum import Enum

from colorama import init

from aigpy.pathHelper import mkdirs

init(autoreset=True)


def isInputYes(string: str) -> bool:
    """Check the input string == yes|y|Yes|Y"""
    if string is not None:
        if str(string).lower() in ['yes', 'y']:
            return True
    return False


def printW(desc: str, wrap: bool = True):
    """Print desc width wrap or not"""
    if not wrap:
        sys.stdout.write(desc)
    else:
        print(desc)


def inputInt(desc: str, default: int) -> int:
    try:
        printW(desc, False)
        string = input()
        return int(string)
    except ValueError:
        return default


def inputFloat(desc: str, default: int) -> float:
    try:
        printW(desc, False)
        string = input()
        return float(string)
    except ValueError:
        return default


def inputPath(desc: str, ignore: str = "") -> str:
    """
    Input path

    return:
    - path: existed or mkdirs success
    """
    printW(desc, False)
    path = input()
    if path == ignore:
        return ignore
    if os.path.isdir(path) or mkdirs(path):
        return path
    return ""


def inputLimit(desc: str, limit: list) -> str:
    """
    Input limit

    return:
    - path: input string in limit list
    - none
    """
    printW(desc, False)
    string = input()
    if string in limit:
        return string
    return None


def isInArgv(string):
    array = sys.argv
    if array is None or len(array) <= 0:
        return False

    array.pop(0)
    if string in array:
        return True
    return False


class TextColor(Enum):
    """Font color"""
    Black = 30
    Blue = 34
    Green = 32
    Red = 31
    Yellow = 33
    White = 37


class BackgroundColor(Enum):
    Black = 40
    Blue = 44
    Green = 42
    Red = 41
    Yellow = 43
    White = 47


def __getColorString__(color: TextColor, bgColor: BackgroundColor, text: str):
    if color is None and bgColor is None:
        return text

    array = []
    if color is not None:
        array.append(str(color.value))
    if bgColor is not None:
        array.append(str(bgColor.value))
    return "\033[" + ';'.join(array) + 'm' + str(text) + "\033[0m"


def green(text):
    return __getColorString__(TextColor.Green, None, text)


def blue(text):
    return __getColorString__(TextColor.Blue, None, text)


def red(text):
    return __getColorString__(TextColor.Red, None, text)


def yellow(text):
    return __getColorString__(TextColor.Yellow, None, text)


def colorPrint(desc: str, textColor: TextColor = None, bgColor: BackgroundColor = None):
    """Print color string

    - textColor (TextColor): font color
    - bgColor (BackgroundColor): background color

    """
    string = __getColorString__(textColor, bgColor, desc)
    printW(string, False)


def printInfo(desc: str):
    print(blue("Info: ") + desc)


def printErr(desc: str):
    print(red("Err: ") + desc)


def printSuccess(desc: str):
    print(green("Success: ") + desc)



import aigpy.LockHelper as lock
import aigpy.cmdHelper as cmd
import aigpy.convertHelper as convert
import aigpy.dictHelper as adict
import aigpy.downloadHelper as download
import aigpy.ffmpegHelper as ffmpeg
import aigpy.fileHelper as file
import aigpy.m3u8Helper as m3u8
import aigpy.modelHelper as model
import aigpy.musicHelp as music
import aigpy.netHelper as net
import aigpy.pathHelper as path
import aigpy.pipHelper as pip
import aigpy.progressHelper as progress
import aigpy.stringHelper as string
import aigpy.systemHelper as system
import aigpy.tagHelper as tag
import aigpy.threadHelper as thread
import aigpy.timeHelp as time
import aigpy.memoryHelper as memory


#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :   pathHelper.py
@Time    :   2018/12/17
@Author  :   Yaronzz 
@Version :   2.0
@Contact :   yaronhuang@foxmail.com
@Desc    :  
"""

import os
import shutil


def getTmpPath(basePath: str) -> str:
    """Get tmp path name like 'Tmp1'"""
    basePath = basePath.replace("\\", "/").strip().rstrip("/")
    count = 0
    path = basePath + '/Tmp' + str(count)
    while os.path.exists(path):
        count = count + 1
        path = basePath + '/Tmp' + str(count)
    return path


def mkdirs(path: str) -> bool:
    path = path.replace("\\", "/")
    path = path.strip()
    path = path.rstrip("/")
    try:
        if not os.path.exists(path):
            os.makedirs(path)
            return True
    except:
        return False
    return True


def remove(path: str) -> bool:
    """Remove file or dir"""
    try:
        if os.path.exists(path) is False:
            return True
        if os.path.isfile(path) is True:
            os.remove(path)
        if os.path.isdir(path) is True:
            shutil.rmtree(path)
        return True
    except:
        return False


def copyFile(srcfile: str, dstfile: str) -> bool:
    if not os.path.isfile(srcfile):
        return False

    path, name = os.path.split(dstfile)
    if not os.path.exists(path):
        mkdirs(path)
    shutil.copyfile(srcfile, dstfile)
    return True


def replaceLimitChar(path: str, newChar: str) -> str:
    if path is None:
        return ""
    if newChar is None:
        newChar = ''
    path = path.replace(':', newChar)
    path = path.replace('/', newChar)
    path = path.replace('?', newChar)
    path = path.replace('<', newChar)
    path = path.replace('>', newChar)
    path = path.replace('|', newChar)
    path = path.replace('\\', newChar)
    path = path.replace('*', newChar)
    path = path.replace('\"', newChar)
    path = path.replace('\n', '')
    path = path.replace('\t', '')
    path = path.rstrip('.')
    path = path.strip(' ')
    return path


def getDirName(filepath: str) -> str:
    """e:/test/file.txt --> e:/test/"""
    filepath = filepath.replace('\\', '/')
    index = filepath.rfind('/')
    if index < 0:
        return './'
    return filepath[0:index + 1]


def getFileName(filepath: str) -> str:
    """e:/test/file.txt --> file.txt"""
    filepath = filepath.replace('\\', '/')
    index = filepath.rfind('/')
    if index < 0:
        return filepath
    return filepath[index + 1:len(filepath)]


def getFileNameWithoutExtension(filepath: str) -> str:
    """e:/test/file.txt --> file"""
    filepath = getFileName(filepath)
    index = filepath.rfind('.')
    if index < 0:
        return filepath
    return filepath[0:index]


def getFileExtension(filepath: str) -> str:
    """e:/test/file.txt --> .txt"""
    filepath = getFileName(filepath)
    index = filepath.rfind('.')
    if index < 0:
        return
    return filepath[index:len(filepath)]


def getSize(path: str) -> int:
    try:
        size = 0
        if os.path.isdir(path) is False:
            return size

        for root, dirs, files in os.walk(path):
            size += sum([os.path.getsize(os.path.join(root, name)) for name in files])
        return size
    except:
        return 0


def getFiles(path: str) -> list:
    try:
        ret = []
        if os.path.isdir(path) is False:
            return ret

        for root, dirs, files in os.walk(path):
            root = root.replace('\\', '/')
            for item in files:
                ret.append(root + '/' + item)
        return ret
    except:
        return []



from setuptools import setup, find_packages

setup(
    name='aigpy',
    version='2022.06.15.1',
    license="MIT Licence",
    description="Python Common Tool",

    author='Yaronzz',
    author_email="yaronhuang@foxmail.com",

    packages=find_packages(),
    platforms="any",
    include_package_data=True,
    install_requires=["requests", "colorama", "mutagen"],
)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-

"""
@File    :   netHelper.py
@Time    :   2018/12/17
@Author  :   Yaronzz 
@Version :   2.0
@Contact :   yaronhuang@foxmail.com
@Desc    :  
"""

import json
import os
import re
import socket
from urllib.request import urlopen

import requests

from aigpy.memoryHelper import unitFix, Unit, convert
from aigpy.pathHelper import getDirName, mkdirs
from aigpy.progressHelper import ProgressTool


def getIpStatus(host: str, port: int, timeouts: int = 1) -> bool:
    """Check the ip status"""
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.settimeout(timeouts)
        s.connect((host, port))
        s.close()
        return True
    except:
        return False


def getIP():
    """Get self-ip"""
    text = requests.get("http://txt.go.sohu.com/ip/soip").text
    ip = re.findall(r'\d+.\d+.\d+.\d+', text)[0]
    return ip


def ignoreCertificate():
    import ssl
    ssl._create_default_https_context = ssl._create_unverified_context


def getResult(code=0, msg='', data=''):
    ret = {'code': code, 'errmsg': msg, 'data': data}
    return json.dumps(ret)


def getSize(url: str) -> int:
    try:
        response = urlopen(url)
        info = response.info()
        dic = dict(info)
        length = dic['Content-Length']
        return int(length)
    except:
        return -1


def getUrlsSize(urls: list) -> (int, list):
    """Get urls size

    Args:
        urls (list): urls array

    Returns:
        [int]: total size
        [list]: urls size
    """
    totalSize = 0
    array = []
    for item in urls:
        size = getSize(item)
        totalSize += size
        if size < 0:
            return -1, []
        array.append(size)
    return totalSize, array


def downloadString(url: str, timeouts=(3.05, 27)):
    try:
        response = requests.get(url, timeouts)
        return response.content
    except:
        return None


def downloadJson(url: str, timeouts=(3.05, 27)):
    try:
        response = requests.get(url, timeouts)
        info = json.loads(response.content)
        return info
    except:
        return None


def downloadFileByUrls(urls: list, fileName, stimeout=3.05, showProgress=False):
    if os.access(fileName, 0):
        os.remove(fileName)

    progress = None
    if showProgress:
        progress = ProgressTool(len(urls), 10)

    for item in urls:
        ret, ex = downloadFile(item, fileName, stimeout, False, append=True)
        if ret is not True or ex is not None:
            return False
        if showProgress:
            progress.step()
    return True


def downloadFile(url: str, fileName: str, stimeout=3.05, showProgress: bool = False, append: bool = False):
    try:
        response = urlopen(url, timeout=stimeout)

        totalSize = response.length
        fileSize, unit = unitFix(totalSize, Unit.BYTE, Unit.MB)
        if showProgress:
            progress = ProgressTool(fileSize, 15, unit=unit.name)

        # mkdir
        path = getDirName(fileName)
        mkdirs(path)

        curcount = 0
        chunksize = 16 * 1024
        mode = 'wb' if not append else 'ab'
        with open(fileName, mode) as f:
            while True:
                chunk = response.read(chunksize)
                curcount += len(chunk)
                if showProgress:
                    progress.setCurCount(convert(curcount, Unit.BYTE, unit))
                f.write(chunk)
                if curcount >= totalSize:
                    break
            return True, None
    except Exception as e:
        return False, e



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  githubHelper.py
@Date    :  2022/02/07
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :  
"""

import requests

class GistFile(object):
    def __init__(self, name, url, content) -> None:
        self.name = name
        self.url = url
        self.content = content

def getGistFiles(gistId):
    array = []
    try:
        respond = requests.get(f'https://api.github.com/gists/{gistId}')
        if respond.status_code == 200:
            files = respond.json()['files']
            for key in files:
                item = files[key]
                array.append(GistFile(item['filename'], item['raw_url'], item['content']))
        return array
    except:
        return array



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :  LockHelper.py
@Date    :  2020/11/16
@Author  :  Yaronzz
@Version :  1.0
@Contact :  yaronhuang@foxmail.com
@Desc    :  
"""

import threading


class RWLock(object):
    def __init__(self):
        self.rlock = threading.Lock()
        self.wlock = threading.Lock()
        self.reader = 0

    def write_acquire(self):
        self.wlock.acquire()

    def write_release(self):
        self.wlock.release()

    def read_acquire(self):
        self.rlock.acquire()
        self.reader += 1
        if self.reader == 1:
            self.wlock.acquire()
        self.rlock.release()

    def read_release(self):
        self.rlock.acquire()
        self.reader -= 1
        if self.reader == 0:
            self.wlock.release()
        self.rlock.release()
